@@ -1,7 +1,7 @@
 
 # $Author: ddumont $
-# $Date: 2008-06-18 18:42:58 +0200 (Wed, 18 Jun 2008) $
-# $Revision: 699 $
+# $Date: 2008-10-13 16:40:22 +0200 (Mon, 13 Oct 2008) $
+# $Revision: 775 $
 
 #    Copyright (c) 2007,2008 Dominique Dumont.
 #
@@ -53,7 +53,7 @@ use Config::Model::Tk::HashEditor ;
 use Config::Model::Tk::NodeViewer ;
 
 
-$VERSION = sprintf "1.%04d", q$Revision: 699 $ =~ /(\d+)/;
+$VERSION = '1.203' ;
 
 Construct Tk::Widget 'ConfigModelUI';
 
@@ -831,20 +831,22 @@ sub edit_copy {
 
     my @selected = @_ ? @_ : $tkt -> info('selection');
 
-    print "edit_copy @selected\n";
+    #print "edit_copy @selected\n";
     my @res ;
 
     foreach my $selection (@selected) {
 	my $data_ref = $tkt->infoData($selection);
 
 	my $cfg_elt = $data_ref->[1] ;
-	print "edit_copy '",$cfg_elt->location, 
-	      "' type '", $cfg_elt->get_type,"'\n";
+	my $type = $cfg_elt->get_type ;
+	my $cfg_class = $type eq 'node' ? $cfg_elt->config_class_name : '';
+	#print "edit_copy '",$cfg_elt->location, "' type '$type' class '$cfg_class'\n";
 
 	push  @res, [ $cfg_elt->element_name,
 		      $cfg_elt->index_value ,
 		      $cfg_elt->composite_name,
-		      $cfg_elt->get_type,
+		      $type,
+		      $cfg_class ,
 		      $cfg_elt->dump_as_data()] ;
     }
 
@@ -861,7 +863,7 @@ sub edit_paste {
 
     my @selected = @_ ? @_ : $tkt -> info('selection');
 
-    print "edit_paste in @selected\n";
+    #print "edit_paste in @selected\n";
     my @res ;
 
     my $selection = $selected[0];
@@ -869,17 +871,19 @@ sub edit_paste {
     my $data_ref = $tkt->infoData($selection);
 
     my $cfg_elt = $data_ref->[1] ;
-    print "edit_paste '",$cfg_elt->location, 
-          "' type '", $cfg_elt->get_type,"'\n";
-    my $t_type = $cfg_elt->get_type ;
-    my $t_name = $cfg_elt->element_name ;
+    #print "edit_paste '",$cfg_elt->location, "' type '", $cfg_elt->get_type,"'\n";
+    my $t_type  = $cfg_elt->get_type ;
+    my $t_class = $t_type eq 'node' ? $cfg_elt->config_class_name : '';
+    my $t_name  = $cfg_elt->element_name ;
     my $cut_buf = $cw->{cut_buffer} || [] ;
 
     foreach my $data (@$cut_buf) {
-	my ($name,$index,$composite,$type, $dump) = @$data;
-	print "from composite name '$composite' type $type\n";
-	print "t_name '$t_name' t_type '$t_type'\n";
-	if ($name eq $t_name and $type eq $t_type) {
+	my ($name,$index,$composite,$type, $cfg_class, $dump) = @$data;
+	#print "from composite name '$composite' type $type\n";
+	#print "t_name '$t_name' t_type '$t_type'  class '$t_class'\n";
+	if (   ($name eq $t_name and $type eq $t_type )
+	    or $t_class eq $cfg_class
+	   ) {
 	   $cfg_elt->load_data($dump) ;
 	}
 	elsif (($t_type eq 'hash' or $t_type eq 'list') and defined $index) {
